'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getSnapshotSuite = exports.getFolder = exports.start = exports.configure = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _lodash = require('lodash.debounce');

var _lodash2 = _interopRequireDefault(_lodash);

var _timm = require('timm');

var _storyboard = require('storyboard');

var _slash = require('slash');

var _slash2 = _interopRequireDefault(_slash);

var _serializer = require('../serializer');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var FOLDER_PATH_ATTR = '__folderPath';

var _config = void 0;
var _folderDict = {};
var _watchers = null;

var configure = function configure(newConfig) {
  if (!_config) {
    _config = (0, _timm.clone)(newConfig);
    return;
  }
  _config = (0, _timm.merge)(_config, newConfig);
};

var start = function start() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$story = _ref.story,
      story = _ref$story === undefined ? _storyboard.mainStory : _ref$story;

  return Promise.resolve().then(function () {
    return loadCommonCss(story);
  }).then(function () {
    return loadAllSnapshots(story);
  }).then(function () {
    if (_config.watch) watchStart(story);
  }).then(function () {
    return broadcastSignal();
  });
};

// ---------------------------------
// Snapshots
// ---------------------------------
var _snapshotSuiteDict = {};

var loadAllSnapshots = function loadAllSnapshots() {
  var story = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _storyboard.mainStory;

  var childStory = story.child({ src: 'extractor', title: 'Refresh snapshots' });
  return Promise.resolve().then(function () {
    return (0, _globby2.default)(_config.snapshotPatterns);
  }).then(function (filePaths) {
    childStory.info('extractor', 'Reading snapshot files...');
    var commonCss = getCommonCss();
    _snapshotSuiteDict = {};
    filePaths.forEach(function (filePath) {
      loadSuite(filePath, commonCss, childStory);
    });
    buildFolderDict(childStory);
    childStory.debug('extractor', 'Snapshot tree:', { attach: _folderDict });
    childStory.close();
  }).catch(function (err) {
    childStory.close();
    throw err;
  });
};

var updateSnapshotCss = function updateSnapshotCss() {
  var story = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _storyboard.mainStory;

  var commonCss = getCommonCss();
  Object.keys(_snapshotSuiteDict).forEach(function (filePath) {
    var suite = _snapshotSuiteDict[filePath];
    var suiteCss = getSuiteCss(filePath, story);
    Object.keys(suite).forEach(function (id) {
      if (id === FOLDER_PATH_ATTR) return;
      var snapshot = suite[id];
      var css = suiteCss != null ? (0, _timm.addLast)(commonCss, suiteCss) : commonCss;
      snapshot.css = css;
    });
  });
};

var loadSuite = function loadSuite(filePath, commonCss, story) {
  story.info('extractor', 'Processing ' + _storyboard.chalk.cyan.bold(filePath) + '...');
  var absPath = _path2.default.resolve(process.cwd(), filePath);
  var rawSnapshots = loadSnapshot(absPath);
  var suiteCss = getSuiteCss(filePath, story);
  var suite = {};
  Object.keys(rawSnapshots).forEach(function (id) {
    var snapshot = rawSnapshots[id];

    var _snapshot$split = snapshot.split(_serializer.HTML_PREVIEW_SEPARATOR),
        _snapshot$split2 = _slicedToArray(_snapshot$split, 2),
        snap = _snapshot$split2[0],
        html = _snapshot$split2[1];

    var css = suiteCss != null ? (0, _timm.addLast)(commonCss, suiteCss) : commonCss;
    suite[id] = { id: id, snap: snap, html: html, css: css };
  });
  story.debug('extractor', 'Found ' + Object.keys(rawSnapshots).length + ' snapshots');
  var finalFilePath = '-/' + filePath.normalize();
  suite[FOLDER_PATH_ATTR] = _path2.default.dirname(finalFilePath);
  _snapshotSuiteDict[finalFilePath] = suite;
};

var loadSnapshot = function loadSnapshot(absPath) {
  /* eslint-disable global-require, import/no-dynamic-require */
  delete require.cache[require.resolve(absPath)];
  return require(absPath);
  /* eslint-enable global-require */
};

// ---------------------------------
// CSS
// ---------------------------------
var _commonCss = [];

var getCommonCss = function getCommonCss() {
  return _commonCss;
};

var loadCommonCss = function loadCommonCss() {
  var story = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _storyboard.mainStory;

  story.info('extractor', 'Extracting common CSS...');
  return Promise.resolve().then(function () {
    return (0, _globby2.default)(_config.cssPatterns);
  }).then(function (cssPaths) {
    _commonCss = cssPaths.map(function (cssPath) {
      story.info('extractor', 'Processing ' + _storyboard.chalk.cyan.bold(cssPath) + '...');
      return _fs2.default.readFileSync(cssPath, 'utf8');
    });
  });
};

var getSuiteCss = function getSuiteCss(filePath, story) {
  var suiteCss = void 0;
  try {
    var cssPath = getCssPathForSuite(filePath);
    story.debug('extractor', 'Trying to read ' + _storyboard.chalk.cyan.bold(cssPath) + '...');
    suiteCss = _fs2.default.readFileSync(cssPath, 'utf8');
    story.info('extractor', 'Found custom CSS in ' + _storyboard.chalk.cyan.bold(cssPath));
  } catch (err) {/* no file exists */}
  return suiteCss;
};

var getCssPathForSuite = function getCssPathForSuite(filePath) {
  var _path$parse = _path2.default.parse(filePath),
      dir = _path$parse.dir,
      name = _path$parse.name;

  return _path2.default.join(dir, name + '.css');
};

// ---------------------------------
// Folder dictionary
// ---------------------------------
var buildFolderDict = function buildFolderDict(story) {
  story.info('extractor', 'Building folder tree...');
  var nextDict = {};

  // Sort file paths to simplify tree generation
  var filePaths = Object.keys(_snapshotSuiteDict).sort();

  // Create root node
  var curFolderPath = '-';
  var curFolder = {
    folderPath: '-',
    filePaths: [],
    parentFolderPath: null,
    childrenFolderPaths: []
  };
  nextDict['-'] = curFolder;

  story.debug('extractor', 'Snapshot tree:', { attach: nextDict });

  // Process all file paths
  filePaths.forEach(function (filePath) {
    story.debug('extractor', 'File: ' + filePath);
    var folderPath = _path2.default.dirname(filePath);
    if (folderPath === curFolderPath) {
      curFolder.filePaths.push(filePath);
    } else {
      var parentFolderPath = folderPath;
      var fFound = false;
      while (!fFound) {
        if (parentFolderPath === '-') break;
        parentFolderPath = _path2.default.dirname(parentFolderPath);
        if (nextDict[parentFolderPath]) {
          fFound = true;
          break;
        }
      }
      if (!fFound) throw new Error('Error building path tree');
      nextDict[parentFolderPath].childrenFolderPaths.push(folderPath);
      curFolder = nextDict[folderPath] = {
        folderPath: folderPath,
        filePaths: [filePath],
        parentFolderPath: parentFolderPath,
        childrenFolderPaths: []
      };
      curFolderPath = folderPath;
    }
  });
  _folderDict = nextDict;
};

// ---------------------------------
// Watching and real-time
// ---------------------------------
var watchStart = function watchStart(story) {
  if (_watchers) return;
  _watchers = {};
  // const glob = _config.snapshotPatterns.concat(_config.cssPatterns);
  var options = {
    ignored: /[\/\\]\./,
    ignoreInitial: true
  };
  _watchers.css = _chokidar2.default.watch(_config.cssPatterns, options);
  _watchers.snap = _chokidar2.default.watch(_config.snapshotPatterns, options);
  ['change', 'add', 'unlink'].forEach(function (type) {
    // $FlowFixMe
    _watchers.css.on(type, function (filePath) {
      cssWatchEvent(type, filePath);
    });
    // $FlowFixMe
    _watchers.snap.on(type, function (filePath) {
      snapWatchEvent(type, filePath);
    });
  });
  story.info('extractor', 'Started watching over snapshot and CSS files');
};

// const watchStop = (story: StoryT) => {
//   if (_watchers == null) return;
//   _watchers.close();
//   _watchers = null;
//   story.info('extractor', 'Stopped file watcher');
// };

var cssWatchEvent = function cssWatchEvent(type, filePath0) {
  var filePath = (0, _slash2.default)(filePath0);
  _storyboard.mainStory.debug('extractor', 'CSS watch fired: ' + (_storyboard.chalk.bold(type.toUpperCase()) + ' ' + _storyboard.chalk.cyan.bold(filePath)));
  debouncedCssRefresh();
};

var debouncedCssRefresh = (0, _lodash2.default)(function () {
  return Promise.resolve().then(function () {
    return loadCommonCss();
  }).then(function () {
    return updateSnapshotCss();
  }).then(function () {
    return broadcastSignal();
  });
}, 300);

var snapWatchEvent = function snapWatchEvent(type, filePath0) {
  var filePath = (0, _slash2.default)(filePath0);
  _storyboard.mainStory.debug('extractor', 'Snapshot watch fired: ' + (_storyboard.chalk.bold(type.toUpperCase()) + ' ' + _storyboard.chalk.cyan.bold(filePath)));
  Promise.resolve().then(function () {
    var commonCss = getCommonCss();
    switch (type) {
      case 'change':
        return loadSuite(filePath, commonCss, _storyboard.mainStory);
      case 'add':
        return Promise.resolve().then(function () {
          return loadSuite(filePath, commonCss, _storyboard.mainStory);
        }).then(function () {
          return buildFolderDict(_storyboard.mainStory);
        });
      case 'unlink':
        return Promise.resolve().then(function () {
          delete _snapshotSuiteDict['-/' + filePath.normalize()];
        }).then(function () {
          return buildFolderDict(_storyboard.mainStory);
        });
      default:
        break;
    }
    return null;
  }).then(function () {
    return broadcastSignal();
  });
};

var broadcastSignal = function broadcastSignal() {
  if (!_config.socketioServer) return;
  _config.socketioServer.emit('REFRESH');
};

// ---------------------------------
// Others
// ---------------------------------
var getFolder = function getFolder(folderPath) {
  return _folderDict[folderPath.normalize()];
};
var getSnapshotSuite = function getSnapshotSuite(filePath) {
  return _snapshotSuiteDict[filePath.normalize()];
};

// =============================================
// Public API
// =============================================
exports.configure = configure;
exports.start = start;
exports.getFolder = getFolder;
exports.getSnapshotSuite = getSnapshotSuite;